@@ -6,7 +6,7 @@ import socket
 import sys
 import threading
 import time
-from Queue import PriorityQueue
+from Queue import PriorityQueue, Empty
 from collections import deque
 
 from hazelcast.connection import Connection, BUFFER_SIZE
@@ -34,7 +34,7 @@ class AsyncoreReactor(object):
         Future._threading_locals.is_reactor_thread = True
         while self._is_live:
             try:
-                asyncore.loop(count=1000, timeout=0.01, map=self._map)
+                asyncore.loop(count=10, timeout=0.01, map=self._map)
                 self._check_timers()
             except select.error as err:
                 # TODO: parse error type to catch only error "9"
@@ -55,7 +55,10 @@ class AsyncoreReactor(object):
                 return
 
             if timer.check_timer(now):
-                self._timers.get_nowait()
+                try:
+                    self._timers.get_nowait()
+                except Empty:
+                    pass
             else:
                 return
 
@@ -143,9 +146,12 @@ class AsyncoreConnection(Connection, asyncore.dispatcher):
 
     def handle_error(self):
         error = sys.exc_info()[1]
-        if error.errno != errno.EAGAIN and error.errno != errno.EDEADLK:
-            self.logger.exception("Received error")
-            self.close(IOError(error))
+        if sys.exc_info()[0] is socket.error:
+            if error.errno != errno.EAGAIN and error.errno != errno.EDEADLK:
+                self.logger.exception("Received error")
+                self.close(IOError(error))
+        else:
+            self.logger.warning("Received unexpected error: " + error)
 
     def readable(self):
         return not self._closed and self.sent_protocol_bytes
