@@ -1,26 +1,31 @@
+import binascii
 import cPickle
 from datetime import datetime
 from time import time
 
 from bits import *
 from hazelcast.serialization.api import StreamSerializer
-# from hazelcast.serialization.base import HazelcastSerializationError
+from hazelcast.serialization.base import HazelcastSerializationError
 from hazelcast.serialization.serialization_const import *
 
 
+class BaseSerializer(StreamSerializer):
+    def destroy(self):
+        pass
+
+
 # DEFAULT SERIALIZERS
-class NoneSerializer(StreamSerializer):
+class NoneSerializer(BaseSerializer):
     def read(self, inp):
         return None
 
-    def write(self, out, obj):
-        pass
+    # "write(self, out, obj)" is never called so not implemented here
 
     def get_type_id(self):
         return CONSTANT_TYPE_NULL
 
 
-class BooleanSerializer(StreamSerializer):
+class BooleanSerializer(BaseSerializer):
     def read(self, inp):
         return inp.read_boolean()
 
@@ -31,7 +36,7 @@ class BooleanSerializer(StreamSerializer):
         return CONSTANT_TYPE_BOOLEAN
 
 
-class ByteSerializer(StreamSerializer):
+class ByteSerializer(BaseSerializer):
     def read(self, inp):
         return inp.read_byte()
 
@@ -42,18 +47,17 @@ class ByteSerializer(StreamSerializer):
         return CONSTANT_TYPE_BYTE
 
 
-class CharSerializer(StreamSerializer):
+class CharSerializer(BaseSerializer):
     def read(self, inp):
         return inp.read_char()
 
-    def write(self, out, obj):
-        out.write_char(obj)
+    # "write(self, out, obj)" is never called so not implemented here
 
     def get_type_id(self):
         return CONSTANT_TYPE_CHAR
 
 
-class ShortSerializer(StreamSerializer):
+class ShortSerializer(BaseSerializer):
     def read(self, inp):
         return inp.read_short()
 
@@ -64,7 +68,7 @@ class ShortSerializer(StreamSerializer):
         return CONSTANT_TYPE_SHORT
 
 
-class IntegerSerializer(StreamSerializer):
+class IntegerSerializer(BaseSerializer):
     def read(self, inp):
         return inp.read_int()
 
@@ -78,7 +82,7 @@ class IntegerSerializer(StreamSerializer):
         return CONSTANT_TYPE_INTEGER
 
 
-class LongSerializer(StreamSerializer):
+class LongSerializer(BaseSerializer):
     def read(self, inp):
         return inp.read_long()
 
@@ -92,7 +96,7 @@ class LongSerializer(StreamSerializer):
         return CONSTANT_TYPE_LONG
 
 
-class FloatSerializer(StreamSerializer):
+class FloatSerializer(BaseSerializer):
     def read(self, inp):
         return inp.read_float()
 
@@ -103,7 +107,7 @@ class FloatSerializer(StreamSerializer):
         return CONSTANT_TYPE_FLOAT
 
 
-class DoubleSerializer(StreamSerializer):
+class DoubleSerializer(BaseSerializer):
     def read(self, inp):
         return inp.read_double()
 
@@ -114,7 +118,7 @@ class DoubleSerializer(StreamSerializer):
         return CONSTANT_TYPE_DOUBLE
 
 
-class StringSerializer(StreamSerializer):
+class StringSerializer(BaseSerializer):
     def read(self, inp):
         return inp.read_utf()
 
@@ -126,107 +130,98 @@ class StringSerializer(StreamSerializer):
 
 
 # ARRAY SERIALIZERS
-class BooleanArraySerializer(StreamSerializer):
+class BooleanArraySerializer(BaseSerializer):
     def read(self, inp):
         return inp.read_boolean_array()
 
-    def write(self, out, obj):
-        out.write_boolean_array(obj)
+    # "write(self, out, obj)" is never called so not implemented here
 
     def get_type_id(self):
         return CONSTANT_TYPE_BOOLEAN_ARRAY
 
 
-class ByteArraySerializer(StreamSerializer):
+class ByteArraySerializer(BaseSerializer):
     def read(self, inp):
         return inp.read_byte_array()
 
-    def write(self, out, obj):
-        out.write_byte_array(obj)
+    # "write(self, out, obj)" is never called so not implemented here
 
     def get_type_id(self):
         return CONSTANT_TYPE_BYTE_ARRAY
 
 
-class CharArraySerializer(StreamSerializer):
+class CharArraySerializer(BaseSerializer):
     def read(self, inp):
         return inp.read_char_array()
 
-    def write(self, out, obj):
-        out.write_char_array(obj)
+    # "write(self, out, obj)" is never called so not implemented here
 
     def get_type_id(self):
         return CONSTANT_TYPE_CHAR_ARRAY
 
 
-class ShortArraySerializer(StreamSerializer):
+class ShortArraySerializer(BaseSerializer):
     def read(self, inp):
         return inp.read_short_array()
 
-    def write(self, out, obj):
-        out.write_short_array(obj)
+    # "write(self, out, obj)" is never called so not implemented here
 
     def get_type_id(self):
         return CONSTANT_TYPE_SHORT_ARRAY
 
 
-class IntegerArraySerializer(StreamSerializer):
+class IntegerArraySerializer(BaseSerializer):
     def read(self, inp):
         return inp.read_int_array()
 
-    def write(self, out, obj):
-        out.write_int_array(obj)
+    # "write(self, out, obj)" is never called so not implemented here
 
     def get_type_id(self):
         return CONSTANT_TYPE_INTEGER_ARRAY
 
 
-class LongArraySerializer(StreamSerializer):
+class LongArraySerializer(BaseSerializer):
     def read(self, inp):
         return inp.read_long_array()
 
-    def write(self, out, obj):
-        out.write_long_array(obj)
+    # "write(self, out, obj)" is never called so not implemented here
 
     def get_type_id(self):
         return CONSTANT_TYPE_LONG_ARRAY
 
 
-class FloatArraySerializer(StreamSerializer):
+class FloatArraySerializer(BaseSerializer):
     def read(self, inp):
         return inp.read_float_array()
 
-    def write(self, out, obj):
-        out.write_float_array(obj)
+    # "write(self, out, obj)" is never called so not implemented here
 
     def get_type_id(self):
         return CONSTANT_TYPE_FLOAT_ARRAY
 
 
-class DoubleArraySerializer(StreamSerializer):
+class DoubleArraySerializer(BaseSerializer):
     def read(self, inp):
         return inp.read_double_array()
 
-    def write(self, out, obj):
-        out.write_double_array(obj)
+    # "write(self, out, obj)" is never called so not implemented here
 
     def get_type_id(self):
         return CONSTANT_TYPE_DOUBLE_ARRAY
 
 
-class StringArraySerializer(StreamSerializer):
+class StringArraySerializer(BaseSerializer):
     def read(self, inp):
         return inp.read_utf_array()
 
-    def write(self, out, obj):
-        out.write_utf_array(obj)
+    # "write(self, out, obj)" is never called so not implemented here
 
     def get_type_id(self):
         return CONSTANT_TYPE_STRING_ARRAY
 
 
 # EXTENSIONS
-class DateTimeSerializer(StreamSerializer):
+class DateTimeSerializer(BaseSerializer):
     def read(self, inp):
         long_time = inp.read_long()
         return datetime.fromtimestamp(long_time / 1000.0)
@@ -239,18 +234,41 @@ class DateTimeSerializer(StreamSerializer):
         return JAVA_DEFAULT_TYPE_DATE
 
 
-class BigIntegerSerializer(StreamSerializer):
+class BigIntegerSerializer(BaseSerializer):
     def read(self, inp):
-        raise NotImplementedError("Big integer numbers not supported")
+        length = inp.read_int()
+        if length == NULL_ARRAY_LENGTH:
+            return None
+        result = bytearray(length)
+        if length > 0:
+            inp.read_into(result, 0, length)
+        if result[0] & 0x80:
+            neg = bytearray()
+            for c in result:
+                neg.append(c ^ 0xFF)
+            return -1 * int(binascii.hexlify(neg), 16) - 1
+        return int(binascii.hexlify(result), 16)
 
     def write(self, out, obj):
-        raise NotImplementedError("Big integer numbers not supported")
+        the_big_int = -obj-1 if obj < 0 else obj
+        end_index = -1 if type(obj) == long else None
+        hex_str = hex(the_big_int)[2:end_index]
+        if len(hex_str) % 2 == 1:
+            prefix = '0' # "f" if obj < 0 else "0"
+            hex_str = prefix + hex_str
+        num_array = bytearray(binascii.unhexlify(bytearray(hex_str)))
+        if obj < 0:
+            neg = bytearray()
+            for c in num_array:
+                neg.append(c ^ 0xFF)
+            num_array = neg
+        out.write_byte_array(num_array)
 
     def get_type_id(self):
         return JAVA_DEFAULT_TYPE_BIG_INTEGER
 
 
-class BigDecimalSerializer(StreamSerializer):
+class BigDecimalSerializer(BaseSerializer):
     def read(self, inp):
         raise NotImplementedError("Big decimal numbers not supported")
 
@@ -261,7 +279,7 @@ class BigDecimalSerializer(StreamSerializer):
         return JAVA_DEFAULT_TYPE_BIG_DECIMAL
 
 
-class JavaClassSerializer(StreamSerializer):
+class JavaClassSerializer(BaseSerializer):
     def read(self, inp):
         return inp.read_utf()
 
@@ -272,7 +290,7 @@ class JavaClassSerializer(StreamSerializer):
         return JAVA_DEFAULT_TYPE_CLASS
 
 
-class JavaEnumSerializer(StreamSerializer):
+class JavaEnumSerializer(BaseSerializer):
     def read(self, inp):
         """
         :param inp:
@@ -289,7 +307,7 @@ class JavaEnumSerializer(StreamSerializer):
         return JAVA_DEFAULT_TYPE_ENUM
 
 
-class ArrayListSerializer(StreamSerializer):
+class ArrayListSerializer(BaseSerializer):
     def read(self, inp):
         size = inp.read_int()
         if size > NULL_ARRAY_LENGTH:
@@ -306,7 +324,7 @@ class ArrayListSerializer(StreamSerializer):
         return JAVA_DEFAULT_TYPE_ARRAY_LIST
 
 
-class LinkedListSerializer(StreamSerializer):
+class LinkedListSerializer(BaseSerializer):
     def read(self, inp):
         size = inp.read_int()
         if size > NULL_ARRAY_LENGTH:
@@ -320,7 +338,7 @@ class LinkedListSerializer(StreamSerializer):
         return JAVA_DEFAULT_TYPE_LINKED_LIST
 
 
-class PythonObjectSerializer(StreamSerializer):
+class PythonObjectSerializer(BaseSerializer):
     def read(self, inp):
         str = inp.read_utf().encode()
         return cPickle.loads(str)
@@ -332,7 +350,7 @@ class PythonObjectSerializer(StreamSerializer):
         return PYTHON_TYPE_PICKLE
 
 
-class IdentifiedDataSerializer(StreamSerializer):
+class IdentifiedDataSerializer(BaseSerializer):
     def __init__(self, factories):
         self._factories = factories
 
@@ -351,7 +369,6 @@ class IdentifiedDataSerializer(StreamSerializer):
 
         factory = self._factories.get(factory_id, None)
         if factory is None:
-            from hazelcast.serialization.base import HazelcastSerializationError
             raise HazelcastSerializationError("No DataSerializerFactory registered for namespace: {}".format(factory_id))
         identified = factory.get(class_id, None)
         if identified is None:
